#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# PKCS#9: Selected Attribute Types (Version 2.0)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc2985.txt
#

from pyasn1.type import (
    char,
    constraint,
    namedtype,
    namedval,
    opentype,
    tag,
    univ,
    useful,
)

from pyasn1_modules import rfc5280, rfc5652, rfc5958, rfc7292


def _OID(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


MAX = float("inf")


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

Attribute = rfc5280.Attribute

EmailAddress = rfc5280.EmailAddress

Extensions = rfc5280.Extensions

Time = rfc5280.Time

X520countryName = rfc5280.X520countryName

X520SerialNumber = rfc5280.X520SerialNumber


# Imports from RFC 5652

ContentInfo = rfc5652.ContentInfo

ContentType = rfc5652.ContentType

Countersignature = rfc5652.Countersignature

MessageDigest = rfc5652.MessageDigest

SignerInfo = rfc5652.SignerInfo

SigningTime = rfc5652.SigningTime


# Imports from RFC 5958

EncryptedPrivateKeyInfo = rfc5958.EncryptedPrivateKeyInfo


# Imports from RFC 7292

PFX = rfc7292.PFX


# TODO:
# Need a place to import PKCS15Token; it does not yet appear in an RFC


# SingleAttribute is the same as Attribute in RFC 5280, except that the
# attrValues SET must have one and only one member


class AttributeType(univ.ObjectIdentifier):
    pass


class AttributeValue(univ.Any):
    pass


class AttributeValues(univ.SetOf):
    pass


AttributeValues.componentType = AttributeValue()


class SingleAttributeValues(univ.SetOf):
    pass


SingleAttributeValues.componentType = AttributeValue()


class SingleAttribute(univ.Sequence):
    pass


SingleAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("type", AttributeType()),
    namedtype.NamedType(
        "values",
        AttributeValues().subtype(sizeSpec=constraint.ValueSizeConstraint(1, 1)),
        openType=opentype.OpenType("type", rfc5280.certificateAttributesMap),
    ),
)


# CMSAttribute is the same as Attribute in RFC 5652, and CMSSingleAttribute
# is the companion where the attrValues SET must have one and only one member

CMSAttribute = rfc5652.Attribute


class CMSSingleAttribute(univ.Sequence):
    pass


CMSSingleAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("attrType", AttributeType()),
    namedtype.NamedType(
        "attrValues",
        AttributeValues().subtype(sizeSpec=constraint.ValueSizeConstraint(1, 1)),
        openType=opentype.OpenType("attrType", rfc5652.cmsAttributesMap),
    ),
)


# DirectoryString is the same as RFC 5280, except the length is limited to 255


class DirectoryString(univ.Choice):
    pass


DirectoryString.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, 255)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, 255)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, 255)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 255)),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 255)),
    ),
)


# PKCS9String is DirectoryString with an additional choice of IA5String,
# and the SIZE is limited to 255


class PKCS9String(univ.Choice):
    pass


PKCS9String.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "ia5String",
        char.IA5String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 255)),
    ),
    namedtype.NamedType("directoryString", DirectoryString()),
)


# Upper Bounds

pkcs_9_ub_pkcs9String = univ.Integer(255)

pkcs_9_ub_challengePassword = univ.Integer(pkcs_9_ub_pkcs9String)

pkcs_9_ub_emailAddress = univ.Integer(pkcs_9_ub_pkcs9String)

pkcs_9_ub_friendlyName = univ.Integer(pkcs_9_ub_pkcs9String)

pkcs_9_ub_match = univ.Integer(pkcs_9_ub_pkcs9String)

pkcs_9_ub_signingDescription = univ.Integer(pkcs_9_ub_pkcs9String)

pkcs_9_ub_unstructuredAddress = univ.Integer(pkcs_9_ub_pkcs9String)

pkcs_9_ub_unstructuredName = univ.Integer(pkcs_9_ub_pkcs9String)


ub_name = univ.Integer(32768)

pkcs_9_ub_placeOfBirth = univ.Integer(ub_name)

pkcs_9_ub_pseudonym = univ.Integer(ub_name)


# Object Identifier Arcs

ietf_at = _OID(1, 3, 6, 1, 5, 5, 7, 9)

id_at = _OID(2, 5, 4)

pkcs_9 = _OID(1, 2, 840, 113549, 1, 9)

pkcs_9_mo = _OID(pkcs_9, 0)

smime = _OID(pkcs_9, 16)

certTypes = _OID(pkcs_9, 22)

crlTypes = _OID(pkcs_9, 23)

pkcs_9_oc = _OID(pkcs_9, 24)

pkcs_9_at = _OID(pkcs_9, 25)

pkcs_9_sx = _OID(pkcs_9, 26)

pkcs_9_mr = _OID(pkcs_9, 27)


# Object Identifiers for Syntaxes for use with LDAP-accessible directories

pkcs_9_sx_pkcs9String = _OID(pkcs_9_sx, 1)

pkcs_9_sx_signingTime = _OID(pkcs_9_sx, 2)


# Object Identifiers for object classes

pkcs_9_oc_pkcsEntity = _OID(pkcs_9_oc, 1)

pkcs_9_oc_naturalPerson = _OID(pkcs_9_oc, 2)


# Object Identifiers for matching rules

pkcs_9_mr_caseIgnoreMatch = _OID(pkcs_9_mr, 1)

pkcs_9_mr_signingTimeMatch = _OID(pkcs_9_mr, 2)


# PKCS #7 PDU

pkcs_9_at_pkcs7PDU = _OID(pkcs_9_at, 5)

pKCS7PDU = Attribute()
pKCS7PDU["type"] = pkcs_9_at_pkcs7PDU
pKCS7PDU["values"][0] = ContentInfo()


# PKCS #12 token

pkcs_9_at_userPKCS12 = _OID(2, 16, 840, 1, 113730, 3, 1, 216)

userPKCS12 = Attribute()
userPKCS12["type"] = pkcs_9_at_userPKCS12
userPKCS12["values"][0] = PFX()


# PKCS #15 token

pkcs_9_at_pkcs15Token = _OID(pkcs_9_at, 1)

# TODO: Once PKCS15Token can be imported, this can be included
#
# pKCS15Token = Attribute()
# userPKCS12['type'] = pkcs_9_at_pkcs15Token
# userPKCS12['values'][0] = PKCS15Token()


# PKCS #8 encrypted private key information

pkcs_9_at_encryptedPrivateKeyInfo = _OID(pkcs_9_at, 2)

encryptedPrivateKeyInfo = Attribute()
encryptedPrivateKeyInfo["type"] = pkcs_9_at_encryptedPrivateKeyInfo
encryptedPrivateKeyInfo["values"][0] = EncryptedPrivateKeyInfo()


# Electronic-mail address

pkcs_9_at_emailAddress = rfc5280.id_emailAddress

emailAddress = Attribute()
emailAddress["type"] = pkcs_9_at_emailAddress
emailAddress["values"][0] = EmailAddress()


# Unstructured name

pkcs_9_at_unstructuredName = _OID(pkcs_9, 2)

unstructuredName = Attribute()
unstructuredName["type"] = pkcs_9_at_unstructuredName
unstructuredName["values"][0] = PKCS9String()


# Unstructured address

pkcs_9_at_unstructuredAddress = _OID(pkcs_9, 8)

unstructuredAddress = Attribute()
unstructuredAddress["type"] = pkcs_9_at_unstructuredAddress
unstructuredAddress["values"][0] = DirectoryString()


# Date of birth

pkcs_9_at_dateOfBirth = _OID(ietf_at, 1)

dateOfBirth = SingleAttribute()
dateOfBirth["type"] = pkcs_9_at_dateOfBirth
dateOfBirth["values"][0] = useful.GeneralizedTime()


# Place of birth

pkcs_9_at_placeOfBirth = _OID(ietf_at, 2)

placeOfBirth = SingleAttribute()
placeOfBirth["type"] = pkcs_9_at_placeOfBirth
placeOfBirth["values"][0] = DirectoryString()


# Gender


class GenderString(char.PrintableString):
    pass


GenderString.subtypeSpec = constraint.ValueSizeConstraint(1, 1)
GenderString.subtypeSpec = constraint.SingleValueConstraint("M", "F", "m", "f")


pkcs_9_at_gender = _OID(ietf_at, 3)

gender = SingleAttribute()
gender["type"] = pkcs_9_at_gender
gender["values"][0] = GenderString()


# Country of citizenship

pkcs_9_at_countryOfCitizenship = _OID(ietf_at, 4)

countryOfCitizenship = Attribute()
countryOfCitizenship["type"] = pkcs_9_at_countryOfCitizenship
countryOfCitizenship["values"][0] = X520countryName()


#  Country of residence

pkcs_9_at_countryOfResidence = _OID(ietf_at, 5)

countryOfResidence = Attribute()
countryOfResidence["type"] = pkcs_9_at_countryOfResidence
countryOfResidence["values"][0] = X520countryName()


# Pseudonym

id_at_pseudonym = _OID(2, 5, 4, 65)

pseudonym = Attribute()
pseudonym["type"] = id_at_pseudonym
pseudonym["values"][0] = DirectoryString()


# Serial number

id_at_serialNumber = rfc5280.id_at_serialNumber

serialNumber = Attribute()
serialNumber["type"] = id_at_serialNumber
serialNumber["values"][0] = X520SerialNumber()


# Content type

pkcs_9_at_contentType = rfc5652.id_contentType

contentType = CMSSingleAttribute()
contentType["attrType"] = pkcs_9_at_contentType
contentType["attrValues"][0] = ContentType()


# Message digest

pkcs_9_at_messageDigest = rfc5652.id_messageDigest

messageDigest = CMSSingleAttribute()
messageDigest["attrType"] = pkcs_9_at_messageDigest
messageDigest["attrValues"][0] = MessageDigest()


# Signing time

pkcs_9_at_signingTime = rfc5652.id_signingTime

signingTime = CMSSingleAttribute()
signingTime["attrType"] = pkcs_9_at_signingTime
signingTime["attrValues"][0] = SigningTime()


# Random nonce


class RandomNonce(univ.OctetString):
    pass


RandomNonce.subtypeSpec = constraint.ValueSizeConstraint(4, MAX)


pkcs_9_at_randomNonce = _OID(pkcs_9_at, 3)

randomNonce = CMSSingleAttribute()
randomNonce["attrType"] = pkcs_9_at_randomNonce
randomNonce["attrValues"][0] = RandomNonce()


# Sequence number


class SequenceNumber(univ.Integer):
    pass


SequenceNumber.subtypeSpec = constraint.ValueRangeConstraint(1, MAX)


pkcs_9_at_sequenceNumber = _OID(pkcs_9_at, 4)

sequenceNumber = CMSSingleAttribute()
sequenceNumber["attrType"] = pkcs_9_at_sequenceNumber
sequenceNumber["attrValues"][0] = SequenceNumber()


# Countersignature

pkcs_9_at_counterSignature = rfc5652.id_countersignature

counterSignature = CMSAttribute()
counterSignature["attrType"] = pkcs_9_at_counterSignature
counterSignature["attrValues"][0] = Countersignature()


# Challenge password

pkcs_9_at_challengePassword = _OID(pkcs_9, 7)

challengePassword = SingleAttribute()
challengePassword["type"] = pkcs_9_at_challengePassword
challengePassword["values"][0] = DirectoryString()


# Extension request


class ExtensionRequest(Extensions):
    pass


pkcs_9_at_extensionRequest = _OID(pkcs_9, 14)

extensionRequest = SingleAttribute()
extensionRequest["type"] = pkcs_9_at_extensionRequest
extensionRequest["values"][0] = ExtensionRequest()


# Extended-certificate attributes (deprecated)


class AttributeSet(univ.SetOf):
    pass


AttributeSet.componentType = Attribute()


pkcs_9_at_extendedCertificateAttributes = _OID(pkcs_9, 9)

extendedCertificateAttributes = SingleAttribute()
extendedCertificateAttributes["type"] = pkcs_9_at_extendedCertificateAttributes
extendedCertificateAttributes["values"][0] = AttributeSet()


# Friendly name


class FriendlyName(char.BMPString):
    pass


FriendlyName.subtypeSpec = constraint.ValueSizeConstraint(1, pkcs_9_ub_friendlyName)


pkcs_9_at_friendlyName = _OID(pkcs_9, 20)

friendlyName = SingleAttribute()
friendlyName["type"] = pkcs_9_at_friendlyName
friendlyName["values"][0] = FriendlyName()


# Local key identifier

pkcs_9_at_localKeyId = _OID(pkcs_9, 21)

localKeyId = SingleAttribute()
localKeyId["type"] = pkcs_9_at_localKeyId
localKeyId["values"][0] = univ.OctetString()


# Signing description

pkcs_9_at_signingDescription = _OID(pkcs_9, 13)

signingDescription = CMSSingleAttribute()
signingDescription["attrType"] = pkcs_9_at_signingDescription
signingDescription["attrValues"][0] = DirectoryString()


# S/MIME capabilities


class SMIMECapability(AlgorithmIdentifier):
    pass


class SMIMECapabilities(univ.SequenceOf):
    pass


SMIMECapabilities.componentType = SMIMECapability()


pkcs_9_at_smimeCapabilities = _OID(pkcs_9, 15)

smimeCapabilities = CMSSingleAttribute()
smimeCapabilities["attrType"] = pkcs_9_at_smimeCapabilities
smimeCapabilities["attrValues"][0] = SMIMECapabilities()


# Certificate Attribute Map

_certificateAttributesMapUpdate = {
    # Attribute types for use with the "pkcsEntity" object class
    pkcs_9_at_pkcs7PDU: ContentInfo(),
    pkcs_9_at_userPKCS12: PFX(),
    # TODO: Once PKCS15Token can be imported, this can be included
    # pkcs_9_at_pkcs15Token: PKCS15Token(),
    pkcs_9_at_encryptedPrivateKeyInfo: EncryptedPrivateKeyInfo(),
    # Attribute types for use with the "naturalPerson" object class
    pkcs_9_at_emailAddress: EmailAddress(),
    pkcs_9_at_unstructuredName: PKCS9String(),
    pkcs_9_at_unstructuredAddress: DirectoryString(),
    pkcs_9_at_dateOfBirth: useful.GeneralizedTime(),
    pkcs_9_at_placeOfBirth: DirectoryString(),
    pkcs_9_at_gender: GenderString(),
    pkcs_9_at_countryOfCitizenship: X520countryName(),
    pkcs_9_at_countryOfResidence: X520countryName(),
    id_at_pseudonym: DirectoryString(),
    id_at_serialNumber: X520SerialNumber(),
    # Attribute types for use with PKCS #10 certificate requests
    pkcs_9_at_challengePassword: DirectoryString(),
    pkcs_9_at_extensionRequest: ExtensionRequest(),
    pkcs_9_at_extendedCertificateAttributes: AttributeSet(),
}

rfc5280.certificateAttributesMap.update(_certificateAttributesMapUpdate)


# CMS Attribute Map

# Note: pkcs_9_at_smimeCapabilities is not included in the map because
#       the definition in RFC 5751 is preferred, which produces the same
#       encoding, but it allows different parameters for SMIMECapability
#       and AlgorithmIdentifier.

_cmsAttributesMapUpdate = {
    # Attribute types for use in PKCS #7 data (a.k.a. CMS)
    pkcs_9_at_contentType: ContentType(),
    pkcs_9_at_messageDigest: MessageDigest(),
    pkcs_9_at_signingTime: SigningTime(),
    pkcs_9_at_randomNonce: RandomNonce(),
    pkcs_9_at_sequenceNumber: SequenceNumber(),
    pkcs_9_at_counterSignature: Countersignature(),
    # Attributes for use in PKCS #12 "PFX" PDUs or PKCS #15 tokens
    pkcs_9_at_friendlyName: FriendlyName(),
    pkcs_9_at_localKeyId: univ.OctetString(),
    pkcs_9_at_signingDescription: DirectoryString(),
    # pkcs_9_at_smimeCapabilities: SMIMECapabilities(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Stanisław Pitucha with asn1ate tool.
# Modified by Russ Housley to add a maps for CMC Control Attributes
#   and CMC Content Types for use with opentypes.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Certificate Management over CMS (CMC) Updates
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6402.txt
#
from pyasn1.type import (
    char,
    constraint,
    namedtype,
    namedval,
    opentype,
    tag,
    univ,
    useful,
)

from pyasn1_modules import rfc4211, rfc5280, rfc5652

MAX = float("inf")


def _buildOid(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


# Since CMS Attributes and CMC Controls both use 'attrType', one map is used
cmcControlAttributesMap = rfc5652.cmsAttributesMap


class ChangeSubjectName(univ.Sequence):
    pass


ChangeSubjectName.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("subject", rfc5280.Name()),
    namedtype.OptionalNamedType("subjectAlt", rfc5280.GeneralNames()),
)


class AttributeValue(univ.Any):
    pass


class CMCStatus(univ.Integer):
    pass


CMCStatus.namedValues = namedval.NamedValues(
    ("success", 0),
    ("failed", 2),
    ("pending", 3),
    ("noSupport", 4),
    ("confirmRequired", 5),
    ("popRequired", 6),
    ("partial", 7),
)


class PendInfo(univ.Sequence):
    pass


PendInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("pendToken", univ.OctetString()),
    namedtype.NamedType("pendTime", useful.GeneralizedTime()),
)

bodyIdMax = univ.Integer(4294967295)


class BodyPartID(univ.Integer):
    pass


BodyPartID.subtypeSpec = constraint.ValueRangeConstraint(0, bodyIdMax)


class BodyPartPath(univ.SequenceOf):
    pass


BodyPartPath.componentType = BodyPartID()
BodyPartPath.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class BodyPartReference(univ.Choice):
    pass


BodyPartReference.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyPartID", BodyPartID()),
    namedtype.NamedType("bodyPartPath", BodyPartPath()),
)


class CMCFailInfo(univ.Integer):
    pass


CMCFailInfo.namedValues = namedval.NamedValues(
    ("badAlg", 0),
    ("badMessageCheck", 1),
    ("badRequest", 2),
    ("badTime", 3),
    ("badCertId", 4),
    ("unsupportedExt", 5),
    ("mustArchiveKeys", 6),
    ("badIdentity", 7),
    ("popRequired", 8),
    ("popFailed", 9),
    ("noKeyReuse", 10),
    ("internalCAError", 11),
    ("tryLater", 12),
    ("authDataFail", 13),
)


class CMCStatusInfoV2(univ.Sequence):
    pass


CMCStatusInfoV2.componentType = namedtype.NamedTypes(
    namedtype.NamedType("cMCStatus", CMCStatus()),
    namedtype.NamedType("bodyList", univ.SequenceOf(componentType=BodyPartReference())),
    namedtype.OptionalNamedType("statusString", char.UTF8String()),
    namedtype.OptionalNamedType(
        "otherInfo",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType("failInfo", CMCFailInfo()),
                namedtype.NamedType("pendInfo", PendInfo()),
                namedtype.NamedType(
                    "extendedFailInfo",
                    univ.Sequence(
                        componentType=namedtype.NamedTypes(
                            namedtype.NamedType("failInfoOID", univ.ObjectIdentifier()),
                            namedtype.NamedType("failInfoValue", AttributeValue()),
                        )
                    ),
                ),
            )
        ),
    ),
)


class GetCRL(univ.Sequence):
    pass


GetCRL.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerName", rfc5280.Name()),
    namedtype.OptionalNamedType("cRLName", rfc5280.GeneralName()),
    namedtype.OptionalNamedType("time", useful.GeneralizedTime()),
    namedtype.OptionalNamedType("reasons", rfc5280.ReasonFlags()),
)

id_pkix = _buildOid(1, 3, 6, 1, 5, 5, 7)

id_cmc = _buildOid(id_pkix, 7)

id_cmc_batchResponses = _buildOid(id_cmc, 29)

id_cmc_popLinkWitness = _buildOid(id_cmc, 23)


class PopLinkWitnessV2(univ.Sequence):
    pass


PopLinkWitnessV2.componentType = namedtype.NamedTypes(
    namedtype.NamedType("keyGenAlgorithm", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("macAlgorithm", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("witness", univ.OctetString()),
)

id_cmc_popLinkWitnessV2 = _buildOid(id_cmc, 33)

id_cmc_identityProofV2 = _buildOid(id_cmc, 34)

id_cmc_revokeRequest = _buildOid(id_cmc, 17)

id_cmc_recipientNonce = _buildOid(id_cmc, 7)


class ControlsProcessed(univ.Sequence):
    pass


ControlsProcessed.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyList", univ.SequenceOf(componentType=BodyPartReference()))
)


class CertificationRequest(univ.Sequence):
    pass


CertificationRequest.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "certificationRequestInfo",
        univ.Sequence(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType("version", univ.Integer()),
                namedtype.NamedType("subject", rfc5280.Name()),
                namedtype.NamedType(
                    "subjectPublicKeyInfo",
                    univ.Sequence(
                        componentType=namedtype.NamedTypes(
                            namedtype.NamedType(
                                "algorithm", rfc5280.AlgorithmIdentifier()
                            ),
                            namedtype.NamedType("subjectPublicKey", univ.BitString()),
                        )
                    ),
                ),
                namedtype.NamedType(
                    "attributes",
                    univ.SetOf(componentType=rfc5652.Attribute()).subtype(
                        implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
                    ),
                ),
            )
        ),
    ),
    namedtype.NamedType("signatureAlgorithm", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("signature", univ.BitString()),
)


class TaggedCertificationRequest(univ.Sequence):
    pass


TaggedCertificationRequest.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyPartID", BodyPartID()),
    namedtype.NamedType("certificationRequest", CertificationRequest()),
)


class TaggedRequest(univ.Choice):
    pass


TaggedRequest.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "tcr",
        TaggedCertificationRequest().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "crm",
        rfc4211.CertReqMsg().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "orm",
        univ.Sequence(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType("bodyPartID", BodyPartID()),
                namedtype.NamedType("requestMessageType", univ.ObjectIdentifier()),
                namedtype.NamedType("requestMessageValue", univ.Any()),
            )
        ).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
)

id_cmc_popLinkRandom = _buildOid(id_cmc, 22)

id_cmc_statusInfo = _buildOid(id_cmc, 1)

id_cmc_trustedAnchors = _buildOid(id_cmc, 26)

id_cmc_transactionId = _buildOid(id_cmc, 5)

id_cmc_encryptedPOP = _buildOid(id_cmc, 9)


class PublishTrustAnchors(univ.Sequence):
    pass


PublishTrustAnchors.componentType = namedtype.NamedTypes(
    namedtype.NamedType("seqNumber", univ.Integer()),
    namedtype.NamedType("hashAlgorithm", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType(
        "anchorHashes", univ.SequenceOf(componentType=univ.OctetString())
    ),
)


class RevokeRequest(univ.Sequence):
    pass


RevokeRequest.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerName", rfc5280.Name()),
    namedtype.NamedType("serialNumber", univ.Integer()),
    namedtype.NamedType("reason", rfc5280.CRLReason()),
    namedtype.OptionalNamedType("invalidityDate", useful.GeneralizedTime()),
    namedtype.OptionalNamedType("passphrase", univ.OctetString()),
    namedtype.OptionalNamedType("comment", char.UTF8String()),
)

id_cmc_senderNonce = _buildOid(id_cmc, 6)

id_cmc_authData = _buildOid(id_cmc, 27)


class TaggedContentInfo(univ.Sequence):
    pass


TaggedContentInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyPartID", BodyPartID()),
    namedtype.NamedType("contentInfo", rfc5652.ContentInfo()),
)


class IdentifyProofV2(univ.Sequence):
    pass


IdentifyProofV2.componentType = namedtype.NamedTypes(
    namedtype.NamedType("proofAlgID", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("macAlgId", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("witness", univ.OctetString()),
)


class CMCPublicationInfo(univ.Sequence):
    pass


CMCPublicationInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hashAlg", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType(
        "certHashes", univ.SequenceOf(componentType=univ.OctetString())
    ),
    namedtype.NamedType("pubInfo", rfc4211.PKIPublicationInfo()),
)

id_kp_cmcCA = _buildOid(rfc5280.id_kp, 27)

id_cmc_confirmCertAcceptance = _buildOid(id_cmc, 24)

id_cmc_raIdentityWitness = _buildOid(id_cmc, 35)

id_ExtensionReq = _buildOid(1, 2, 840, 113549, 1, 9, 14)

id_cct = _buildOid(id_pkix, 12)

id_cct_PKIData = _buildOid(id_cct, 2)

id_kp_cmcRA = _buildOid(rfc5280.id_kp, 28)


class CMCStatusInfo(univ.Sequence):
    pass


CMCStatusInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("cMCStatus", CMCStatus()),
    namedtype.NamedType("bodyList", univ.SequenceOf(componentType=BodyPartID())),
    namedtype.OptionalNamedType("statusString", char.UTF8String()),
    namedtype.OptionalNamedType(
        "otherInfo",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType("failInfo", CMCFailInfo()),
                namedtype.NamedType("pendInfo", PendInfo()),
            )
        ),
    ),
)


class DecryptedPOP(univ.Sequence):
    pass


DecryptedPOP.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyPartID", BodyPartID()),
    namedtype.NamedType("thePOPAlgID", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("thePOP", univ.OctetString()),
)

id_cmc_addExtensions = _buildOid(id_cmc, 8)

id_cmc_modCertTemplate = _buildOid(id_cmc, 31)


class TaggedAttribute(univ.Sequence):
    pass


TaggedAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyPartID", BodyPartID()),
    namedtype.NamedType("attrType", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "attrValues",
        univ.SetOf(componentType=AttributeValue()),
        openType=opentype.OpenType("attrType", cmcControlAttributesMap),
    ),
)


class OtherMsg(univ.Sequence):
    pass


OtherMsg.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyPartID", BodyPartID()),
    namedtype.NamedType("otherMsgType", univ.ObjectIdentifier()),
    namedtype.NamedType("otherMsgValue", univ.Any()),
)


class PKIData(univ.Sequence):
    pass


PKIData.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "controlSequence", univ.SequenceOf(componentType=TaggedAttribute())
    ),
    namedtype.NamedType("reqSequence", univ.SequenceOf(componentType=TaggedRequest())),
    namedtype.NamedType(
        "cmsSequence", univ.SequenceOf(componentType=TaggedContentInfo())
    ),
    namedtype.NamedType("otherMsgSequence", univ.SequenceOf(componentType=OtherMsg())),
)


class BodyPartList(univ.SequenceOf):
    pass


BodyPartList.componentType = BodyPartID()
BodyPartList.sizeSpec = constraint.ValueSizeConstraint(1, MAX)

id_cmc_responseBody = _buildOid(id_cmc, 37)


class AuthPublish(BodyPartID):
    pass


class CMCUnsignedData(univ.Sequence):
    pass


CMCUnsignedData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bodyPartPath", BodyPartPath()),
    namedtype.NamedType("identifier", univ.ObjectIdentifier()),
    namedtype.NamedType("content", univ.Any()),
)


class CMCCertId(rfc5652.IssuerAndSerialNumber):
    pass


class PKIResponse(univ.Sequence):
    pass


PKIResponse.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "controlSequence", univ.SequenceOf(componentType=TaggedAttribute())
    ),
    namedtype.NamedType(
        "cmsSequence", univ.SequenceOf(componentType=TaggedContentInfo())
    ),
    namedtype.NamedType("otherMsgSequence", univ.SequenceOf(componentType=OtherMsg())),
)


class ResponseBody(PKIResponse):
    pass


id_cmc_statusInfoV2 = _buildOid(id_cmc, 25)

id_cmc_lraPOPWitness = _buildOid(id_cmc, 11)


class ModCertTemplate(univ.Sequence):
    pass


ModCertTemplate.componentType = namedtype.NamedTypes(
    namedtype.NamedType("pkiDataReference", BodyPartPath()),
    namedtype.NamedType("certReferences", BodyPartList()),
    namedtype.DefaultedNamedType("replace", univ.Boolean().subtype(value=1)),
    namedtype.NamedType("certTemplate", rfc4211.CertTemplate()),
)

id_cmc_regInfo = _buildOid(id_cmc, 18)

id_cmc_identityProof = _buildOid(id_cmc, 3)


class ExtensionReq(univ.SequenceOf):
    pass


ExtensionReq.componentType = rfc5280.Extension()
ExtensionReq.sizeSpec = constraint.ValueSizeConstraint(1, MAX)

id_kp_cmcArchive = _buildOid(rfc5280.id_kp, 28)

id_cmc_publishCert = _buildOid(id_cmc, 30)

id_cmc_dataReturn = _buildOid(id_cmc, 4)


class LraPopWitness(univ.Sequence):
    pass


LraPopWitness.componentType = namedtype.NamedTypes(
    namedtype.NamedType("pkiDataBodyid", BodyPartID()),
    namedtype.NamedType("bodyIds", univ.SequenceOf(componentType=BodyPartID())),
)

id_aa = _buildOid(1, 2, 840, 113549, 1, 9, 16, 2)

id_aa_cmc_unsignedData = _buildOid(id_aa, 34)

id_cmc_getCert = _buildOid(id_cmc, 15)

id_cmc_batchRequests = _buildOid(id_cmc, 28)

id_cmc_decryptedPOP = _buildOid(id_cmc, 10)

id_cmc_responseInfo = _buildOid(id_cmc, 19)

id_cmc_changeSubjectName = _buildOid(id_cmc, 36)


class GetCert(univ.Sequence):
    pass


GetCert.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerName", rfc5280.GeneralName()),
    namedtype.NamedType("serialNumber", univ.Integer()),
)

id_cmc_identification = _buildOid(id_cmc, 2)

id_cmc_queryPending = _buildOid(id_cmc, 21)


class AddExtensions(univ.Sequence):
    pass


AddExtensions.componentType = namedtype.NamedTypes(
    namedtype.NamedType("pkiDataReference", BodyPartID()),
    namedtype.NamedType("certReferences", univ.SequenceOf(componentType=BodyPartID())),
    namedtype.NamedType(
        "extensions", univ.SequenceOf(componentType=rfc5280.Extension())
    ),
)


class EncryptedPOP(univ.Sequence):
    pass


EncryptedPOP.componentType = namedtype.NamedTypes(
    namedtype.NamedType("request", TaggedRequest()),
    namedtype.NamedType("cms", rfc5652.ContentInfo()),
    namedtype.NamedType("thePOPAlgID", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("witnessAlgID", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("witness", univ.OctetString()),
)

id_cmc_getCRL = _buildOid(id_cmc, 16)

id_cct_PKIResponse = _buildOid(id_cct, 3)

id_cmc_controlProcessed = _buildOid(id_cmc, 32)


class NoSignatureValue(univ.OctetString):
    pass


id_ad_cmc = _buildOid(rfc5280.id_ad, 12)

id_alg_noSignature = _buildOid(id_pkix, 6, 2)


# Map of CMC Control OIDs to CMC Control Attributes

_cmcControlAttributesMapUpdate = {
    id_cmc_statusInfo: CMCStatusInfo(),
    id_cmc_statusInfoV2: CMCStatusInfoV2(),
    id_cmc_identification: char.UTF8String(),
    id_cmc_identityProof: univ.OctetString(),
    id_cmc_identityProofV2: IdentifyProofV2(),
    id_cmc_dataReturn: univ.OctetString(),
    id_cmc_transactionId: univ.Integer(),
    id_cmc_senderNonce: univ.OctetString(),
    id_cmc_recipientNonce: univ.OctetString(),
    id_cmc_addExtensions: AddExtensions(),
    id_cmc_encryptedPOP: EncryptedPOP(),
    id_cmc_decryptedPOP: DecryptedPOP(),
    id_cmc_lraPOPWitness: LraPopWitness(),
    id_cmc_getCert: GetCert(),
    id_cmc_getCRL: GetCRL(),
    id_cmc_revokeRequest: RevokeRequest(),
    id_cmc_regInfo: univ.OctetString(),
    id_cmc_responseInfo: univ.OctetString(),
    id_cmc_queryPending: univ.OctetString(),
    id_cmc_popLinkRandom: univ.OctetString(),
    id_cmc_popLinkWitness: univ.OctetString(),
    id_cmc_popLinkWitnessV2: PopLinkWitnessV2(),
    id_cmc_confirmCertAcceptance: CMCCertId(),
    id_cmc_trustedAnchors: PublishTrustAnchors(),
    id_cmc_authData: AuthPublish(),
    id_cmc_batchRequests: BodyPartList(),
    id_cmc_batchResponses: BodyPartList(),
    id_cmc_publishCert: CMCPublicationInfo(),
    id_cmc_modCertTemplate: ModCertTemplate(),
    id_cmc_controlProcessed: ControlsProcessed(),
    id_ExtensionReq: ExtensionReq(),
}

cmcControlAttributesMap.update(_cmcControlAttributesMapUpdate)


# Map of CMC Content Type OIDs to CMC Content Types are added to
# the ones that are in rfc5652.py

_cmsContentTypesMapUpdate = {
    id_cct_PKIData: PKIData(),
    id_cct_PKIResponse: PKIResponse(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to add items from the verified errata.
# Modified by Russ Housley to add maps for use with opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# CMS Firmware Wrapper
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4108.txt
# https://www.rfc-editor.org/errata_search.php?rfc=4108
#


from pyasn1.type import char, constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc5280, rfc5652

MAX = float("inf")


class HardwareSerialEntry(univ.Choice):
    pass


HardwareSerialEntry.componentType = namedtype.NamedTypes(
    namedtype.NamedType("all", univ.Null()),
    namedtype.NamedType("single", univ.OctetString()),
    namedtype.NamedType(
        "block",
        univ.Sequence(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType("low", univ.OctetString()),
                namedtype.NamedType("high", univ.OctetString()),
            )
        ),
    ),
)


class HardwareModules(univ.Sequence):
    pass


HardwareModules.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hwType", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "hwSerialEntries", univ.SequenceOf(componentType=HardwareSerialEntry())
    ),
)


class CommunityIdentifier(univ.Choice):
    pass


CommunityIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("communityOID", univ.ObjectIdentifier()),
    namedtype.NamedType("hwModuleList", HardwareModules()),
)


class PreferredPackageIdentifier(univ.Sequence):
    pass


PreferredPackageIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("fwPkgID", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "verNum",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
    ),
)


class PreferredOrLegacyPackageIdentifier(univ.Choice):
    pass


PreferredOrLegacyPackageIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("preferred", PreferredPackageIdentifier()),
    namedtype.NamedType("legacy", univ.OctetString()),
)


class CurrentFWConfig(univ.Sequence):
    pass


CurrentFWConfig.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("fwPkgType", univ.Integer()),
    namedtype.NamedType("fwPkgName", PreferredOrLegacyPackageIdentifier()),
)


class PreferredOrLegacyStalePackageIdentifier(univ.Choice):
    pass


PreferredOrLegacyStalePackageIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "preferredStaleVerNum",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
    ),
    namedtype.NamedType("legacyStaleVersion", univ.OctetString()),
)


class FirmwarePackageLoadErrorCode(univ.Enumerated):
    pass


FirmwarePackageLoadErrorCode.namedValues = namedval.NamedValues(
    ("decodeFailure", 1),
    ("badContentInfo", 2),
    ("badSignedData", 3),
    ("badEncapContent", 4),
    ("badCertificate", 5),
    ("badSignerInfo", 6),
    ("badSignedAttrs", 7),
    ("badUnsignedAttrs", 8),
    ("missingContent", 9),
    ("noTrustAnchor", 10),
    ("notAuthorized", 11),
    ("badDigestAlgorithm", 12),
    ("badSignatureAlgorithm", 13),
    ("unsupportedKeySize", 14),
    ("signatureFailure", 15),
    ("contentTypeMismatch", 16),
    ("badEncryptedData", 17),
    ("unprotectedAttrsPresent", 18),
    ("badEncryptContent", 19),
    ("badEncryptAlgorithm", 20),
    ("missingCiphertext", 21),
    ("noDecryptKey", 22),
    ("decryptFailure", 23),
    ("badCompressAlgorithm", 24),
    ("missingCompressedContent", 25),
    ("decompressFailure", 26),
    ("wrongHardware", 27),
    ("stalePackage", 28),
    ("notInCommunity", 29),
    ("unsupportedPackageType", 30),
    ("missingDependency", 31),
    ("wrongDependencyVersion", 32),
    ("insufficientMemory", 33),
    ("badFirmware", 34),
    ("unsupportedParameters", 35),
    ("breaksDependency", 36),
    ("otherError", 99),
)


class VendorLoadErrorCode(univ.Integer):
    pass


# Wrapped Firmware Key Unsigned Attribute and Object Identifier

id_aa_wrappedFirmwareKey = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.39")


class WrappedFirmwareKey(rfc5652.EnvelopedData):
    pass


# Firmware Package Information Signed Attribute and Object Identifier

id_aa_firmwarePackageInfo = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.42")


class FirmwarePackageInfo(univ.Sequence):
    pass


FirmwarePackageInfo.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("fwPkgType", univ.Integer()),
    namedtype.OptionalNamedType(
        "dependencies",
        univ.SequenceOf(componentType=PreferredOrLegacyPackageIdentifier()),
    ),
)

FirmwarePackageInfo.sizeSpec = univ.Sequence.sizeSpec + constraint.ValueSizeConstraint(
    1, 2
)


# Community Identifiers Signed Attribute and Object Identifier

id_aa_communityIdentifiers = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.40")


class CommunityIdentifiers(univ.SequenceOf):
    pass


CommunityIdentifiers.componentType = CommunityIdentifier()


# Implemented Compression Algorithms Signed Attribute and Object Identifier

id_aa_implCompressAlgs = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.43")


class ImplementedCompressAlgorithms(univ.SequenceOf):
    pass


ImplementedCompressAlgorithms.componentType = univ.ObjectIdentifier()


# Implemented Cryptographic Algorithms Signed Attribute and Object Identifier

id_aa_implCryptoAlgs = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.38")


class ImplementedCryptoAlgorithms(univ.SequenceOf):
    pass


ImplementedCryptoAlgorithms.componentType = univ.ObjectIdentifier()


# Decrypt Key Identifier Signed Attribute and Object Identifier

id_aa_decryptKeyID = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.37")


class DecryptKeyIdentifier(univ.OctetString):
    pass


# Target Hardware Identifier Signed Attribute and Object Identifier

id_aa_targetHardwareIDs = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.36")


class TargetHardwareIdentifiers(univ.SequenceOf):
    pass


TargetHardwareIdentifiers.componentType = univ.ObjectIdentifier()


# Firmware Package Identifier Signed Attribute and Object Identifier

id_aa_firmwarePackageID = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.35")


class FirmwarePackageIdentifier(univ.Sequence):
    pass


FirmwarePackageIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("name", PreferredOrLegacyPackageIdentifier()),
    namedtype.OptionalNamedType("stale", PreferredOrLegacyStalePackageIdentifier()),
)


# Firmware Package Message Digest Signed Attribute and Object Identifier

id_aa_fwPkgMessageDigest = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.41")


class FirmwarePackageMessageDigest(univ.Sequence):
    pass


FirmwarePackageMessageDigest.componentType = namedtype.NamedTypes(
    namedtype.NamedType("algorithm", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("msgDigest", univ.OctetString()),
)


# Firmware Package Load Error Report Content Type and Object Identifier


class FWErrorVersion(univ.Integer):
    pass


FWErrorVersion.namedValues = namedval.NamedValues(("v1", 1))


id_ct_firmwareLoadError = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.18")


class FirmwarePackageLoadError(univ.Sequence):
    pass


FirmwarePackageLoadError.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("version", FWErrorVersion().subtype(value="v1")),
    namedtype.NamedType("hwType", univ.ObjectIdentifier()),
    namedtype.NamedType("hwSerialNum", univ.OctetString()),
    namedtype.NamedType("errorCode", FirmwarePackageLoadErrorCode()),
    namedtype.OptionalNamedType("vendorErrorCode", VendorLoadErrorCode()),
    namedtype.OptionalNamedType("fwPkgName", PreferredOrLegacyPackageIdentifier()),
    namedtype.OptionalNamedType(
        "config",
        univ.SequenceOf(componentType=CurrentFWConfig()).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


# Firmware Package Load Receipt Content Type and Object Identifier


class FWReceiptVersion(univ.Integer):
    pass


FWReceiptVersion.namedValues = namedval.NamedValues(("v1", 1))


id_ct_firmwareLoadReceipt = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.17")


class FirmwarePackageLoadReceipt(univ.Sequence):
    pass


FirmwarePackageLoadReceipt.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("version", FWReceiptVersion().subtype(value="v1")),
    namedtype.NamedType("hwType", univ.ObjectIdentifier()),
    namedtype.NamedType("hwSerialNum", univ.OctetString()),
    namedtype.NamedType("fwPkgName", PreferredOrLegacyPackageIdentifier()),
    namedtype.OptionalNamedType("trustAnchorKeyID", univ.OctetString()),
    namedtype.OptionalNamedType(
        "decryptKeyID",
        univ.OctetString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


# Firmware Package Content Type and Object Identifier

id_ct_firmwarePackage = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.16")


class FirmwarePkgData(univ.OctetString):
    pass


# Other Name syntax for Hardware Module Name

id_on_hardwareModuleName = univ.ObjectIdentifier("1.3.6.1.5.5.7.8.4")


class HardwareModuleName(univ.Sequence):
    pass


HardwareModuleName.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hwType", univ.ObjectIdentifier()),
    namedtype.NamedType("hwSerialNum", univ.OctetString()),
)


# Map of Attribute Type OIDs to Attributes is added to the
# ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_wrappedFirmwareKey: WrappedFirmwareKey(),
    id_aa_firmwarePackageInfo: FirmwarePackageInfo(),
    id_aa_communityIdentifiers: CommunityIdentifiers(),
    id_aa_implCompressAlgs: ImplementedCompressAlgorithms(),
    id_aa_implCryptoAlgs: ImplementedCryptoAlgorithms(),
    id_aa_decryptKeyID: DecryptKeyIdentifier(),
    id_aa_targetHardwareIDs: TargetHardwareIdentifiers(),
    id_aa_firmwarePackageID: FirmwarePackageIdentifier(),
    id_aa_fwPkgMessageDigest: FirmwarePackageMessageDigest(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)


# Map of Content Type OIDs to Content Types is added to the
# ones that are in rfc5652.py

_cmsContentTypesMapUpdate = {
    id_ct_firmwareLoadError: FirmwarePackageLoadError(),
    id_ct_firmwareLoadReceipt: FirmwarePackageLoadReceipt(),
    id_ct_firmwarePackage: FirmwarePkgData(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)


# Map of Other Name OIDs to Other Name is added to the
# ones that are in rfc5280.py

_anotherNameMapUpdate = {
    id_on_hardwareModuleName: HardwareModuleName(),
}

rfc5280.anotherNameMap.update(_anotherNameMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# SNMPv2c message syntax
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc1901.txt
#
from pyasn1.type import namedtype, namedval, univ


class Message(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "version", univ.Integer(namedValues=namedval.NamedValues(("version-2c", 1)))
        ),
        namedtype.NamedType("community", univ.OctetString()),
        namedtype.NamedType("data", univ.Any()),
    )



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
import unittest

suite = unittest.TestLoader().loadTestsFromNames(
    [
        "tests.test_pem.suite",
        "tests.test_rfc2314.suite",
        "tests.test_rfc2315.suite",
        "tests.test_rfc2437.suite",
        "tests.test_rfc2459.suite",
        "tests.test_rfc2511.suite",
        "tests.test_rfc2560.suite",
        "tests.test_rfc2631.suite",
        "tests.test_rfc2634.suite",
        "tests.test_rfc2876.suite",
        "tests.test_rfc2985.suite",
        "tests.test_rfc2986.suite",
        "tests.test_rfc3058.suite",
        "tests.test_rfc3114.suite",
        "tests.test_rfc3125.suite",
        "tests.test_rfc3161.suite",
        "tests.test_rfc3274.suite",
        "tests.test_rfc3279.suite",
        "tests.test_rfc3280.suite",
        "tests.test_rfc3281.suite",
        "tests.test_rfc3370.suite",
        "tests.test_rfc3447.suite",
        "tests.test_rfc3537.suite",
        "tests.test_rfc3560.suite",
        "tests.test_rfc3565.suite",
        "tests.test_rfc3657.suite",
        "tests.test_rfc3709.suite",
        "tests.test_rfc3739.suite",
        "tests.test_rfc3770.suite",
        "tests.test_rfc3779.suite",
        "tests.test_rfc3820.suite",
        "tests.test_rfc3852.suite",
        "tests.test_rfc4010.suite",
        "tests.test_rfc4043.suite",
        "tests.test_rfc4055.suite",
        "tests.test_rfc4073.suite",
        "tests.test_rfc4108.suite",
        "tests.test_rfc4210.suite",
        "tests.test_rfc4211.suite",
        "tests.test_rfc4334.suite",
        "tests.test_rfc4357.suite",
        "tests.test_rfc4387.suite",
        "tests.test_rfc4476.suite",
        "tests.test_rfc4490.suite",
        "tests.test_rfc4491.suite",
        "tests.test_rfc4683.suite",
        "tests.test_rfc4985.suite",
        "tests.test_rfc5035.suite",
        "tests.test_rfc5083.suite",
        "tests.test_rfc5084.suite",
        "tests.test_rfc5126.suite",
        "tests.test_rfc5208.suite",
        "tests.test_rfc5275.suite",
        "tests.test_rfc5280.suite",
        "tests.test_rfc5480.suite",
        "tests.test_rfc5636.suite",
        "tests.test_rfc5639.suite",
        "tests.test_rfc5649.suite",
        "tests.test_rfc5652.suite",
        "tests.test_rfc5697.suite",
        "tests.test_rfc5751.suite",
        "tests.test_rfc5752.suite",
        "tests.test_rfc5753.suite",
        "tests.test_rfc5755.suite",
        "tests.test_rfc5913.suite",
        "tests.test_rfc5914.suite",
        "tests.test_rfc5915.suite",
        "tests.test_rfc5916.suite",
        "tests.test_rfc5917.suite",
        "tests.test_rfc5924.suite",
        "tests.test_rfc5934.suite",
        "tests.test_rfc5940.suite",
        "tests.test_rfc5958.suite",
        "tests.test_rfc5990.suite",
        "tests.test_rfc6010.suite",
        "tests.test_rfc6019.suite",
        "tests.test_rfc6031.suite",
        "tests.test_rfc6032.suite",
        "tests.test_rfc6120.suite",
        "tests.test_rfc6187.suite",
        "tests.test_rfc6210.suite",
        "tests.test_rfc6211.suite",
        "tests.test_rfc6482.suite",
        "tests.test_rfc6486.suite",
        "tests.test_rfc6487.suite",
        "tests.test_rfc6664.suite",
        "tests.test_rfc6955.suite",
        "tests.test_rfc6960.suite",
        "tests.test_rfc7030.suite",
        "tests.test_rfc7191.suite",
        "tests.test_rfc7229.suite",
        "tests.test_rfc7292.suite",
        "tests.test_rfc7296.suite",
        "tests.test_rfc7508.suite",
        "tests.test_rfc7585.suite",
        "tests.test_rfc7633.suite",
        "tests.test_rfc7773.suite",
        "tests.test_rfc7894.suite",
        "tests.test_rfc7906.suite",
        "tests.test_rfc7914.suite",
        "tests.test_rfc8017.suite",
        "tests.test_rfc8018.suite",
        "tests.test_rfc8103.suite",
        "tests.test_rfc8209.suite",
        "tests.test_rfc8226.suite",
        "tests.test_rfc8358.suite",
        "tests.test_rfc8360.suite",
        "tests.test_rfc8398.suite",
        "tests.test_rfc8410.suite",
        "tests.test_rfc8418.suite",
        "tests.test_rfc8419.suite",
        "tests.test_rfc8479.suite",
        "tests.test_rfc8494.suite",
        "tests.test_rfc8520.suite",
        "tests.test_rfc8619.suite",
        "tests.test_rfc8649.suite",
        "tests.test_rfc8692.suite",
        "tests.test_rfc8696.suite",
        "tests.test_rfc8702.suite",
        "tests.test_rfc8708.suite",
        "tests.test_rfc8769.suite",
    ]
)


if __name__ == "__main__":
    import sys

    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Electronic Signature Policies
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3125.txt
# https://www.rfc-editor.org/errata/eid5901
# https://www.rfc-editor.org/errata/eid5902
#

from pyasn1.type import constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc5280

MAX = float("inf")


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

Attribute = rfc5280.Attribute

AttributeType = rfc5280.AttributeType

AttributeTypeAndValue = rfc5280.AttributeTypeAndValue

AttributeValue = rfc5280.AttributeValue

Certificate = rfc5280.Certificate

CertificateList = rfc5280.CertificateList

DirectoryString = rfc5280.DirectoryString

GeneralName = rfc5280.GeneralName

GeneralNames = rfc5280.GeneralNames

Name = rfc5280.Name

PolicyInformation = rfc5280.PolicyInformation


# Electronic Signature Policies


class CertPolicyId(univ.ObjectIdentifier):
    pass


class AcceptablePolicySet(univ.SequenceOf):
    componentType = CertPolicyId()


class SignPolExtn(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("extnID", univ.ObjectIdentifier()),
        namedtype.NamedType("extnValue", univ.OctetString()),
    )


class SignPolExtensions(univ.SequenceOf):
    componentType = SignPolExtn()


class AlgAndLength(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("algID", univ.ObjectIdentifier()),
        namedtype.OptionalNamedType("minKeyLength", univ.Integer()),
        namedtype.OptionalNamedType("other", SignPolExtensions()),
    )


class AlgorithmConstraints(univ.SequenceOf):
    componentType = AlgAndLength()


class AlgorithmConstraintSet(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "signerAlgorithmConstraints",
            AlgorithmConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "eeCertAlgorithmConstraints",
            AlgorithmConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "caCertAlgorithmConstraints",
            AlgorithmConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "aaCertAlgorithmConstraints",
            AlgorithmConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "tsaCertAlgorithmConstraints",
            AlgorithmConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
            ),
        ),
    )


class AttributeValueConstraints(univ.SequenceOf):
    componentType = AttributeTypeAndValue()


class AttributeTypeConstraints(univ.SequenceOf):
    componentType = AttributeType()


class AttributeConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "attributeTypeConstarints",
            AttributeTypeConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "attributeValueConstarints",
            AttributeValueConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class HowCertAttribute(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ("claimedAttribute", 0), ("certifiedAttribtes", 1), ("either", 2)
    )


class SkipCerts(univ.Integer):
    subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


class PolicyConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "requireExplicitPolicy",
            SkipCerts().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "inhibitPolicyMapping",
            SkipCerts().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class BaseDistance(univ.Integer):
    subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


class GeneralSubtree(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("base", GeneralName()),
        namedtype.DefaultedNamedType(
            "minimum",
            BaseDistance()
            .subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
            .subtype(value=0),
        ),
        namedtype.OptionalNamedType(
            "maximum",
            BaseDistance().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class GeneralSubtrees(univ.SequenceOf):
    componentType = GeneralSubtree()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class NameConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "permittedSubtrees",
            GeneralSubtrees().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "excludedSubtrees",
            GeneralSubtrees().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class PathLenConstraint(univ.Integer):
    subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


class CertificateTrustPoint(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("trustpoint", Certificate()),
        namedtype.OptionalNamedType(
            "pathLenConstraint",
            PathLenConstraint().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "acceptablePolicySet",
            AcceptablePolicySet().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "nameConstraints",
            NameConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "policyConstraints",
            PolicyConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
    )


class CertificateTrustTrees(univ.SequenceOf):
    componentType = CertificateTrustPoint()


class EnuRevReq(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ("clrCheck", 0),
        ("ocspCheck", 1),
        ("bothCheck", 2),
        ("eitherCheck", 3),
        ("noCheck", 4),
        ("other", 5),
    )


class RevReq(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("enuRevReq", EnuRevReq()),
        namedtype.OptionalNamedType("exRevReq", SignPolExtensions()),
    )


class CertRevReq(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("endCertRevReq", RevReq()),
        namedtype.NamedType(
            "caCerts",
            RevReq().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )


class AttributeTrustCondition(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("attributeMandated", univ.Boolean()),
        namedtype.NamedType("howCertAttribute", HowCertAttribute()),
        namedtype.OptionalNamedType(
            "attrCertificateTrustTrees",
            CertificateTrustTrees().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "attrRevReq",
            CertRevReq().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "attributeConstraints",
            AttributeConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
    )


class CMSAttrs(univ.SequenceOf):
    componentType = univ.ObjectIdentifier()


class CertInfoReq(univ.Enumerated):
    namedValues = namedval.NamedValues(("none", 0), ("signerOnly", 1), ("fullPath", 2))


class CertRefReq(univ.Enumerated):
    namedValues = namedval.NamedValues(("signerOnly", 1), ("fullPath", 2))


class DeltaTime(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("deltaSeconds", univ.Integer()),
        namedtype.NamedType("deltaMinutes", univ.Integer()),
        namedtype.NamedType("deltaHours", univ.Integer()),
        namedtype.NamedType("deltaDays", univ.Integer()),
    )


class TimestampTrustCondition(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "ttsCertificateTrustTrees",
            CertificateTrustTrees().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "ttsRevReq",
            CertRevReq().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "ttsNameConstraints",
            NameConstraints().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "cautionPeriod",
            DeltaTime().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "signatureTimestampDelay",
            DeltaTime().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
    )


class SignerRules(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType("externalSignedData", univ.Boolean()),
        namedtype.NamedType("mandatedSignedAttr", CMSAttrs()),
        namedtype.NamedType("mandatedUnsignedAttr", CMSAttrs()),
        namedtype.DefaultedNamedType(
            "mandatedCertificateRef",
            CertRefReq()
            .subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
            .subtype(value="signerOnly"),
        ),
        namedtype.DefaultedNamedType(
            "mandatedCertificateInfo",
            CertInfoReq()
            .subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
            .subtype(value="none"),
        ),
        namedtype.OptionalNamedType(
            "signPolExtensions",
            SignPolExtensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class MandatedUnsignedAttr(CMSAttrs):
    pass


class VerifierRules(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("mandatedUnsignedAttr", MandatedUnsignedAttr()),
        namedtype.OptionalNamedType("signPolExtensions", SignPolExtensions()),
    )


class SignerAndVerifierRules(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signerRules", SignerRules()),
        namedtype.NamedType("verifierRules", VerifierRules()),
    )


class SigningCertTrustCondition(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signerTrustTrees", CertificateTrustTrees()),
        namedtype.NamedType("signerRevReq", CertRevReq()),
    )


class CommitmentTypeIdentifier(univ.ObjectIdentifier):
    pass


class FieldOfApplication(DirectoryString):
    pass


class CommitmentType(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("identifier", CommitmentTypeIdentifier()),
        namedtype.OptionalNamedType(
            "fieldOfApplication",
            FieldOfApplication().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "semantics",
            DirectoryString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class SelectedCommitmentTypes(univ.SequenceOf):
    componentType = univ.Choice(
        componentType=namedtype.NamedTypes(
            namedtype.NamedType("empty", univ.Null()),
            namedtype.NamedType("recognizedCommitmentType", CommitmentType()),
        )
    )


class CommitmentRule(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("selCommitmentTypes", SelectedCommitmentTypes()),
        namedtype.OptionalNamedType(
            "signerAndVeriferRules",
            SignerAndVerifierRules().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "signingCertTrustCondition",
            SigningCertTrustCondition().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "timeStampTrustCondition",
            TimestampTrustCondition().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "attributeTrustCondition",
            AttributeTrustCondition().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "algorithmConstraintSet",
            AlgorithmConstraintSet().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
        namedtype.OptionalNamedType(
            "signPolExtensions",
            SignPolExtensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
            ),
        ),
    )


class CommitmentRules(univ.SequenceOf):
    componentType = CommitmentRule()


class CommonRules(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "signerAndVeriferRules",
            SignerAndVerifierRules().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "signingCertTrustCondition",
            SigningCertTrustCondition().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "timeStampTrustCondition",
            TimestampTrustCondition().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "attributeTrustCondition",
            AttributeTrustCondition().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "algorithmConstraintSet",
            AlgorithmConstraintSet().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
        namedtype.OptionalNamedType(
            "signPolExtensions",
            SignPolExtensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
            ),
        ),
    )


class PolicyIssuerName(GeneralNames):
    pass


class SignPolicyHash(univ.OctetString):
    pass


class SignPolicyId(univ.ObjectIdentifier):
    pass


class SigningPeriod(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("notBefore", useful.GeneralizedTime()),
        namedtype.OptionalNamedType("notAfter", useful.GeneralizedTime()),
    )


class SignatureValidationPolicy(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signingPeriod", SigningPeriod()),
        namedtype.NamedType("commonRules", CommonRules()),
        namedtype.NamedType("commitmentRules", CommitmentRules()),
        namedtype.OptionalNamedType("signPolExtensions", SignPolExtensions()),
    )


class SignPolicyInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signPolicyIdentifier", SignPolicyId()),
        namedtype.NamedType("dateOfIssue", useful.GeneralizedTime()),
        namedtype.NamedType("policyIssuerName", PolicyIssuerName()),
        namedtype.NamedType("fieldOfApplication", FieldOfApplication()),
        namedtype.NamedType("signatureValidationPolicy", SignatureValidationPolicy()),
        namedtype.OptionalNamedType("signPolExtensions", SignPolExtensions()),
    )


class SignaturePolicy(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signPolicyHashAlg", AlgorithmIdentifier()),
        namedtype.NamedType("signPolicyInfo", SignPolicyInfo()),
        namedtype.OptionalNamedType("signPolicyHash", SignPolicyHash()),
    )



#!/usr/bin/env python
#
# Read CMC certificate request with wrappers on stdin, parse each into
# plain text, then build substrate from it
#
import sys

from pyasn1.codec.der import decoder, encoder

from pyasn1_modules import pem, rfc5652, rfc6402

if len(sys.argv) != 1:
    print(
        """Usage:
$ cat cmc_request.pem | %s"""
        % (sys.argv[0],)
    )
    sys.exit(-1)

reqCnt = 0

substrate = pem.readBase64FromFile(sys.stdin)

_, rest = decoder.decode(substrate, asn1Spec=rfc5652.ContentInfo())
assert not rest

next_layer = rfc5652.id_ct_contentInfo
data = substrate
while next_layer:
    if next_layer == rfc5652.id_ct_contentInfo:
        layer, rest = decoder.decode(data, asn1Spec=rfc5652.ContentInfo())
        assert encoder.encode(layer) == data, "wrapper recode fails"
        assert not rest

        print(" * New layer (wrapper):")
        print(layer.prettyPrint())

        next_layer = layer["contentType"]
        data = layer["content"]

    elif next_layer == rfc5652.id_signedData:
        layer, rest = decoder.decode(data, asn1Spec=rfc5652.SignedData())
        assert encoder.encode(layer) == data, "wrapper recode fails"
        assert not rest

        print(" * New layer (wrapper):")
        print(layer.prettyPrint())

        next_layer = layer["encapContentInfo"]["eContentType"]
        data = layer["encapContentInfo"]["eContent"]

    elif next_layer == rfc6402.id_cct_PKIData:
        layer, rest = decoder.decode(data, asn1Spec=rfc6402.PKIData())
        assert encoder.encode(layer) == data, "pkidata recode fails"
        assert not rest

        print(" * New layer (pkidata):")
        print(layer.prettyPrint())

        next_layer = None
        data = None



#
# This file is part of pyasn1-modules software.
#
# Created by Stanisław Pitucha with asn1ate tool.
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Internet X.509 Public Key Infrastructure Certificate Request
# Message Format (CRMF)
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc4211.txt
#
from pyasn1.type import char, constraint, namedtype, namedval, tag, univ

from pyasn1_modules import rfc3280, rfc3852

MAX = float("inf")


def _buildOid(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


id_pkix = _buildOid(1, 3, 6, 1, 5, 5, 7)

id_pkip = _buildOid(id_pkix, 5)

id_regCtrl = _buildOid(id_pkip, 1)


class SinglePubInfo(univ.Sequence):
    pass


SinglePubInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "pubMethod",
        univ.Integer(
            namedValues=namedval.NamedValues(
                ("dontCare", 0), ("x500", 1), ("web", 2), ("ldap", 3)
            )
        ),
    ),
    namedtype.OptionalNamedType("pubLocation", rfc3280.GeneralName()),
)


class UTF8Pairs(char.UTF8String):
    pass


class PKMACValue(univ.Sequence):
    pass


PKMACValue.componentType = namedtype.NamedTypes(
    namedtype.NamedType("algId", rfc3280.AlgorithmIdentifier()),
    namedtype.NamedType("value", univ.BitString()),
)


class POPOSigningKeyInput(univ.Sequence):
    pass


POPOSigningKeyInput.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "authInfo",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType(
                    "sender",
                    rfc3280.GeneralName().subtype(
                        implicitTag=tag.Tag(
                            tag.tagClassContext, tag.tagFormatConstructed, 0
                        )
                    ),
                ),
                namedtype.NamedType("publicKeyMAC", PKMACValue()),
            )
        ),
    ),
    namedtype.NamedType("publicKey", rfc3280.SubjectPublicKeyInfo()),
)


class POPOSigningKey(univ.Sequence):
    pass


POPOSigningKey.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "poposkInput",
        POPOSigningKeyInput().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType("algorithmIdentifier", rfc3280.AlgorithmIdentifier()),
    namedtype.NamedType("signature", univ.BitString()),
)


class Attributes(univ.SetOf):
    pass


Attributes.componentType = rfc3280.Attribute()


class PrivateKeyInfo(univ.Sequence):
    pass


PrivateKeyInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", univ.Integer()),
    namedtype.NamedType("privateKeyAlgorithm", rfc3280.AlgorithmIdentifier()),
    namedtype.NamedType("privateKey", univ.OctetString()),
    namedtype.OptionalNamedType(
        "attributes",
        Attributes().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class EncryptedValue(univ.Sequence):
    pass


EncryptedValue.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "intendedAlg",
        rfc3280.AlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "symmAlg",
        rfc3280.AlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "encSymmKey",
        univ.BitString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.OptionalNamedType(
        "keyAlg",
        rfc3280.AlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.OptionalNamedType(
        "valueHint",
        univ.OctetString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
        ),
    ),
    namedtype.NamedType("encValue", univ.BitString()),
)


class EncryptedKey(univ.Choice):
    pass


EncryptedKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType("encryptedValue", EncryptedValue()),
    namedtype.NamedType(
        "envelopedData",
        rfc3852.EnvelopedData().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class KeyGenParameters(univ.OctetString):
    pass


class PKIArchiveOptions(univ.Choice):
    pass


PKIArchiveOptions.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "encryptedPrivKey",
        EncryptedKey().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "keyGenParameters",
        KeyGenParameters().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "archiveRemGenPrivKey",
        univ.Boolean().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)

id_regCtrl_authenticator = _buildOid(id_regCtrl, 2)

id_regInfo = _buildOid(id_pkip, 2)

id_regInfo_certReq = _buildOid(id_regInfo, 2)


class ProtocolEncrKey(rfc3280.SubjectPublicKeyInfo):
    pass


class Authenticator(char.UTF8String):
    pass


class SubsequentMessage(univ.Integer):
    pass


SubsequentMessage.namedValues = namedval.NamedValues(
    ("encrCert", 0), ("challengeResp", 1)
)


class AttributeTypeAndValue(univ.Sequence):
    pass


AttributeTypeAndValue.componentType = namedtype.NamedTypes(
    namedtype.NamedType("type", univ.ObjectIdentifier()),
    namedtype.NamedType("value", univ.Any()),
)


class POPOPrivKey(univ.Choice):
    pass


POPOPrivKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "thisMessage",
        univ.BitString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "subsequentMessage",
        SubsequentMessage().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "dhMAC",
        univ.BitString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.NamedType(
        "agreeMAC",
        PKMACValue().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
        ),
    ),
    namedtype.NamedType(
        "encryptedKey",
        rfc3852.EnvelopedData().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
        ),
    ),
)


class ProofOfPossession(univ.Choice):
    pass


ProofOfPossession.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "raVerified",
        univ.Null().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "signature",
        POPOSigningKey().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
    namedtype.NamedType(
        "keyEncipherment",
        POPOPrivKey().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
    namedtype.NamedType(
        "keyAgreement",
        POPOPrivKey().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
        ),
    ),
)


class OptionalValidity(univ.Sequence):
    pass


OptionalValidity.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "notBefore",
        rfc3280.Time().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "notAfter",
        rfc3280.Time().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class CertTemplate(univ.Sequence):
    pass


CertTemplate.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "version",
        rfc3280.Version().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "serialNumber",
        univ.Integer().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "signingAlg",
        rfc3280.AlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.OptionalNamedType(
        "issuer",
        rfc3280.Name().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
        ),
    ),
    namedtype.OptionalNamedType(
        "validity",
        OptionalValidity().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
        ),
    ),
    namedtype.OptionalNamedType(
        "subject",
        rfc3280.Name().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
        ),
    ),
    namedtype.OptionalNamedType(
        "publicKey",
        rfc3280.SubjectPublicKeyInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 6)
        ),
    ),
    namedtype.OptionalNamedType(
        "issuerUID",
        rfc3280.UniqueIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
        ),
    ),
    namedtype.OptionalNamedType(
        "subjectUID",
        rfc3280.UniqueIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 8)
        ),
    ),
    namedtype.OptionalNamedType(
        "extensions",
        rfc3280.Extensions().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 9)
        ),
    ),
)


class Controls(univ.SequenceOf):
    pass


Controls.componentType = AttributeTypeAndValue()
Controls.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class CertRequest(univ.Sequence):
    pass


CertRequest.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certReqId", univ.Integer()),
    namedtype.NamedType("certTemplate", CertTemplate()),
    namedtype.OptionalNamedType("controls", Controls()),
)


class CertReqMsg(univ.Sequence):
    pass


CertReqMsg.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certReq", CertRequest()),
    namedtype.OptionalNamedType("popo", ProofOfPossession()),
    namedtype.OptionalNamedType(
        "regInfo", univ.SequenceOf(componentType=AttributeTypeAndValue())
    ),
)


class CertReqMessages(univ.SequenceOf):
    pass


CertReqMessages.componentType = CertReqMsg()
CertReqMessages.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class CertReq(CertRequest):
    pass


id_regCtrl_pkiPublicationInfo = _buildOid(id_regCtrl, 3)


class CertId(univ.Sequence):
    pass


CertId.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuer", rfc3280.GeneralName()),
    namedtype.NamedType("serialNumber", univ.Integer()),
)


class OldCertId(CertId):
    pass


class PKIPublicationInfo(univ.Sequence):
    pass


PKIPublicationInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "action",
        univ.Integer(
            namedValues=namedval.NamedValues(("dontPublish", 0), ("pleasePublish", 1))
        ),
    ),
    namedtype.OptionalNamedType(
        "pubInfos", univ.SequenceOf(componentType=SinglePubInfo())
    ),
)


class EncKeyWithID(univ.Sequence):
    pass


EncKeyWithID.componentType = namedtype.NamedTypes(
    namedtype.NamedType("privateKey", PrivateKeyInfo()),
    namedtype.OptionalNamedType(
        "identifier",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType("string", char.UTF8String()),
                namedtype.NamedType("generalName", rfc3280.GeneralName()),
            )
        ),
    ),
)

id_regCtrl_protocolEncrKey = _buildOid(id_regCtrl, 6)

id_regCtrl_oldCertID = _buildOid(id_regCtrl, 5)

id_smime = _buildOid(1, 2, 840, 113549, 1, 9, 16)


class PBMParameter(univ.Sequence):
    pass


PBMParameter.componentType = namedtype.NamedTypes(
    namedtype.NamedType("salt", univ.OctetString()),
    namedtype.NamedType("owf", rfc3280.AlgorithmIdentifier()),
    namedtype.NamedType("iterationCount", univ.Integer()),
    namedtype.NamedType("mac", rfc3280.AlgorithmIdentifier()),
)

id_regCtrl_regToken = _buildOid(id_regCtrl, 1)

id_regCtrl_pkiArchiveOptions = _buildOid(id_regCtrl, 4)

id_regInfo_utf8Pairs = _buildOid(id_regInfo, 1)

id_ct = _buildOid(id_smime, 1)

id_ct_encKeyWithID = _buildOid(id_ct, 21)


class RegToken(char.UTF8String):
    pass



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import univ

from pyasn1_modules import pem, rfc5652, rfc5958, rfc8410


class PrivateKeyTestCase(unittest.TestCase):
    priv_key_pem_text = """\
MHICAQEwBQYDK2VwBCIEINTuctv5E1hK1bbY8fdp+K06/nwoy/HU++CXqI9EdVhC
oB8wHQYKKoZIhvcNAQkJFDEPDA1DdXJkbGUgQ2hhaXJzgSEAGb9ECWmEzf6FQbrB
Z9w7lshQhqowtrbLDFw4rXAxZuE=
"""

    def setUp(self):
        self.asn1Spec = rfc5958.PrivateKeyInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.priv_key_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(
            rfc8410.id_Ed25519, asn1Object["privateKeyAlgorithm"]["algorithm"]
        )
        self.assertTrue(asn1Object["privateKey"].isValue)
        self.assertEqual("0x0420d4ee", asn1Object["privateKey"].prettyPrint()[0:10])
        self.assertTrue(asn1Object["publicKey"].isValue)
        self.assertEqual("1164575857", asn1Object["publicKey"].prettyPrint()[0:10])
        self.assertEqual(substrate, der_encoder(asn1Object))


class PrivateKeyOpenTypesTestCase(unittest.TestCase):
    asymmetric_key_pkg_pem_text = """\
MIGEBgpghkgBZQIBAk4FoHYwdDByAgEBMAUGAytlcAQiBCDU7nLb+RNYStW22PH3
afitOv58KMvx1Pvgl6iPRHVYQqAfMB0GCiqGSIb3DQEJCRQxDwwNQ3VyZGxlIENo
YWlyc4EhABm/RAlphM3+hUG6wWfcO5bIUIaqMLa2ywxcOK1wMWbh
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.asymmetric_key_pkg_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertIn(rfc5958.id_ct_KP_aKeyPackage, rfc5652.cmsContentTypesMap)

        oneKey = asn1Object["content"][0]

        self.assertEqual(rfc8410.id_Ed25519, oneKey["privateKeyAlgorithm"]["algorithm"])

        pkcs_9_at_friendlyName = univ.ObjectIdentifier("1.2.840.113549.1.9.9.20")

        self.assertEqual(pkcs_9_at_friendlyName, oneKey["attributes"][0]["attrType"])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# RPKI Manifests
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6486.txt
#

from pyasn1.type import char, constraint, namedtype, tag, univ, useful

from pyasn1_modules import rfc5652

MAX = float("inf")


id_smime = univ.ObjectIdentifier("1.2.840.113549.1.9.16")

id_ct = id_smime + (1,)

id_ct_rpkiManifest = id_ct + (26,)


class FileAndHash(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("file", char.IA5String()),
        namedtype.NamedType("hash", univ.BitString()),
    )


class Manifest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "version",
            univ.Integer()
            .subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
            .subtype(value=0),
        ),
        namedtype.NamedType(
            "manifestNumber",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
        ),
        namedtype.NamedType("thisUpdate", useful.GeneralizedTime()),
        namedtype.NamedType("nextUpdate", useful.GeneralizedTime()),
        namedtype.NamedType("fileHashAlg", univ.ObjectIdentifier()),
        namedtype.NamedType(
            "fileList",
            univ.SequenceOf(componentType=FileAndHash()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(0, MAX)
            ),
        ),
    )


# Map of Content Type OIDs to Content Types added to the
# ones that are in rfc5652.py

_cmsContentTypesMapUpdate = {
    id_ct_rpkiManifest: Manifest(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)
